==> plasma-framework_5.28.0.orig.tar.xz: plasma-framework-5.28.0/src/plasma/private/componentinstaller.cpp <==
/*
 *   Copyright 2011 Kevin Kofler <kevin.kofler@chello.at>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "private/componentinstaller_p.h"

#include <QSet>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QWidget>
#include <QLatin1String>
#include <QStringList>

namespace Plasma
{

class ComponentInstallerPrivate
{
public:
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QSet<QString> alreadyPrompted;
#endif
};

class ComponentInstallerSingleton
{
public:
    ComponentInstaller self;
};

Q_GLOBAL_STATIC(ComponentInstallerSingleton, privateComponentInstallerSelf)

ComponentInstaller *ComponentInstaller::self()
{
    return &privateComponentInstallerSelf()->self;
}

ComponentInstaller::ComponentInstaller()
    : d(new ComponentInstallerPrivate)
{
}

ComponentInstaller::~ComponentInstaller()
{
    delete d;
}

void ComponentInstaller::installMissingComponent(const QString &type,
        const QString &name,
        QWidget *parent, bool force)
{
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QString searchString = type + '-' + name;

    if (!force) {
        if (d->alreadyPrompted.contains(searchString)) {
            return;
        }
    }

    d->alreadyPrompted.insert(searchString);

    QDBusInterface packageKit(QStringLiteral("org.freedesktop.PackageKit"),
                              QStringLiteral("/org/freedesktop/PackageKit"),
                              QStringLiteral("org.freedesktop.PackageKit.Modify"));
    // We don't check packageKit.isValid() because the service is activated on
    // demand, so it will show up as "not valid".
    WId wid = 0;
    if (parent) {
        wid = parent->winId();
    }
    QStringList resources;
    resources.append(searchString);
    packageKit.asyncCall(QStringLiteral("InstallResources"), (unsigned int) wid,
                         QLatin1String("plasma-service"), resources, QString());
#else
    Q_UNUSED(type);
    Q_UNUSED(name);
    Q_UNUSED(parent);
    Q_UNUSED(force);
#endif
}

} // namespace Plasma

==> plasma-framework_5.44.0.orig.tar.xz: plasma-framework-5.44.0/src/plasma/private/componentinstaller.cpp <==
/*
 *   Copyright 2011 Kevin Kofler <kevin.kofler@chello.at>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "private/componentinstaller_p.h"

#include <QSet>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QWidget>
#include <QLatin1String>
#include <QStringList>

namespace Plasma
{

class ComponentInstallerPrivate
{
public:
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QSet<QString> alreadyPrompted;
#endif
};

class ComponentInstallerSingleton
{
public:
    ComponentInstaller self;
};

Q_GLOBAL_STATIC(ComponentInstallerSingleton, privateComponentInstallerSelf)

ComponentInstaller *ComponentInstaller::self()
{
    return &privateComponentInstallerSelf()->self;
}

ComponentInstaller::ComponentInstaller()
    : d(new ComponentInstallerPrivate)
{
}

ComponentInstaller::~ComponentInstaller()
{
    delete d;
}

void ComponentInstaller::installMissingComponent(const QString &type,
        const QString &name,
        QWidget *parent, bool force)
{
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QString searchString = type + QLatin1Char('-') + name;

    if (!force) {
        if (d->alreadyPrompted.contains(searchString)) {
            return;
        }
    }

    d->alreadyPrompted.insert(searchString);

    QDBusInterface packageKit(QStringLiteral("org.freedesktop.PackageKit"),
                              QStringLiteral("/org/freedesktop/PackageKit"),
                              QStringLiteral("org.freedesktop.PackageKit.Modify"));
    // We don't check packageKit.isValid() because the service is activated on
    // demand, so it will show up as "not valid".
    WId wid = 0;
    if (parent) {
        wid = parent->winId();
    }
    QStringList resources;
    resources.append(searchString);
    packageKit.asyncCall(QStringLiteral("InstallResources"), (unsigned int) wid,
                         QLatin1String("plasma-service"), resources, QString());
#else
    Q_UNUSED(type);
    Q_UNUSED(name);
    Q_UNUSED(parent);
    Q_UNUSED(force);
#endif
}

} // namespace Plasma
